@@ -536,8 +536,15 @@ int process_chroot_path( ShellOptions_t *opts,
 {
 	char	*temp;		/* to hold the assigned path */
 
-	/* get rid of any old value for chroot path, assign new one */
-	if ( opts->chroot_path ) free(opts->chroot_path);
+	/* 
+	 * older versions of rssh free()'d opts->chroot_path here, but this is
+	 * an error.  If the current config line had an error, that left the
+	 * chroot_path in an invalid state.  We now wait until after the
+	 * new chroot_path is successfully assigned by get_asgn_param() to do
+	 * the free() call.  Waiting also allows us to preserve any previous
+	 * chroot path if there is an error.
+	 */
+
 	if ( !(temp = (char *)malloc(CFG_LINE_LEN + 1)) ){
 		log_msg("fatal error: can't allocate space for chroot path");
 		exit(1);
@@ -547,6 +554,9 @@ int process_chroot_path( ShellOptions_t *opts,
 		free(temp);
 		return FALSE;
 	}
+	
+	/* get rid of any old value for chroot path, assign new one */
+	if ( opts->chroot_path ) free(opts->chroot_path);
 	log_set_priority(LOG_INFO);
 	log_msg("chrooting all users to %s", temp);
 	/* we must not free temp, since opts points to it */
@@ -919,6 +929,18 @@ cleanup:
 		log_msg("allowing sftp to user %s", user);
 		opts->shell_flags |= RSSH_ALLOW_SFTP;
 	}
+	if ( allow_cvs ){
+		log_msg("allowing cvs to user %s", user);
+		opts->shell_flags |= RSSH_ALLOW_CVS;
+	}
+	if ( allow_rdist ){
+		log_msg("allowing rdist to user %s", user);
+		opts->shell_flags |= RSSH_ALLOW_RDIST;
+	}
+	if ( allow_rsync ){
+		log_msg("allowing rsync to user %s", user);
+		opts->shell_flags |= RSSH_ALLOW_RSYNC;
+	}
 	if ( path ){
 		log_msg("chrooting %s to %s", user, path);
 		opts->shell_flags |= RSSH_USE_CHROOT;
