@@ -120,6 +120,42 @@ void fail( int flags, int argc, char **argv )
 }
 
 
+/*
+ * opt_exist() - takes a string representing a command line, and a single
+ *               character representing a command-line option to search for.
+ *               If the option exists in the command line, return TRUE.  This
+ *               function is a little over-zealous about returning a match,
+ *               but in this case that is better than not being zealous
+ *               enough.  And frankly, I don't want to spend the time required
+ *               to get it 100% right.  It's not worth the effort.
+ */
+bool opt_exist(char *cl, char opt)
+{
+	int	i = 0;
+	int	len;
+	char	*token;
+	bool	optstring = FALSE;
+
+
+	len = strlen(cl);
+
+	/* process command line character by character */
+	while ( i < (len - 2) ){
+		if ( cl[i] == ' ' || cl[i] == '\t' ){
+			if ( cl[i+1] == '-' ){ 
+				optstring = TRUE;
+				i+=2;
+			}
+		}
+		if ( cl[i] == opt && optstring ) return TRUE;
+		if ( cl[i] == ' ' || cl[i] == '\t' || cl[i] == '-' ) 
+			optstring = FALSE;
+		i++;
+	}
+	return FALSE;
+}
+
+
 bool check_command( char *cl, ShellOptions_t *opts, char *cmd, int cmdflag )
 {
 	int	cl_len;		/* length of command line */
@@ -161,14 +197,46 @@ char *check_command_line( char *cl, ShellOptions_t *opts )
 
 	if ( check_command(cl, opts, PATH_SFTP_SERVER, RSSH_ALLOW_SFTP) )
 		return PATH_SFTP_SERVER;
-	if ( check_command(cl, opts, PATH_SCP, RSSH_ALLOW_SCP) )
+
+	if ( check_command(cl, opts, PATH_SCP, RSSH_ALLOW_SCP) ){
+		/* filter -S option */
+		if ( opt_exist(cl, 'S') ){
+			fprintf(stderr, "\ninsecure -S option not allowed.");
+			log_msg("insecure -S option in scp command line!");
+			return NULL;
+		}
 		return PATH_SCP;
+	}
+
 	if ( check_command(cl, opts, PATH_CVS, RSSH_ALLOW_CVS) )
 		return PATH_CVS;
-	if ( check_command(cl, opts, PATH_RDIST, RSSH_ALLOW_RDIST) )
+
+	if ( check_command(cl, opts, PATH_RDIST, RSSH_ALLOW_RDIST) ){
+		/* filter -P option */
+		if ( opt_exist(cl, 'P') ){
+			fprintf(stderr, "\ninsecure -P option not allowed.");
+			log_msg("insecure -P option in rdist command line!");
+			return NULL;
+		}
 		return PATH_RDIST;
-	if ( check_command(cl, opts, PATH_RSYNC, RSSH_ALLOW_RSYNC) )
+	}
+
+	if ( check_command(cl, opts, PATH_RSYNC, RSSH_ALLOW_RSYNC) ){
+		/* filter -e option */
+		if ( opt_exist(cl, 'e') ){
+			fprintf(stderr, "\ninsecure -e option not allowed.");
+			log_msg("insecure -e option in rdist command line!");
+			return NULL;
+		}
+		
+		if ( strstr(cl, "--rsh=" ) ){
+			fprintf(stderr, "\ninsecure --rsh= not allowed.");
+			log_msg("insecure --rsh option in rsync command line!");
+			return NULL;
+		}
+
 		return PATH_RSYNC;
+	}
 
 	return NULL;
 }
@@ -252,3 +320,5 @@ int validate_access( const char *temp, bool *allow_sftp, bool *allow_scp,
 	*allow_scp   = temp[4] - '0';
 	return TRUE;
 }
+
+
